#!/bin/bash

# ================================================================================
# Setup variables

# kubectl
SERVICE_ACCOUNT=iot-accelerator-webapp-account
SERVICE_ACCOUNT_ROLE=iot-accelerator-webapp-access-role
NAMESPACE=default


# ================================================================================
# STEP 1: Install required software

# Install kubectl
curl -LO "https://dl.k8s.io/release/$(curl -L -s https://dl.k8s.io/release/stable.txt)/bin/linux/amd64/kubectl"
sudo install -o root -g root -m 0755 kubectl /usr/local/bin/kubectl
kubectl version --client

# ================================================================================
# STEP 2: Create Kubernetes service account and role

# Create a service account
kubectl create serviceaccount $SERVICE_ACCOUNT

# Role for the service account that will be used to access k8s via. REST API
kubectl apply -f - <<!
kind: ClusterRole
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: $SERVICE_ACCOUNT_ROLE
rules:
# For role escalation problem. the REST API should be able to grant permission to the service account service-update so make sure these permissions below > permission setting in REST call
- apiGroups: ["apps", ""] 
  resources: ["pods", "services", "deployments", ]
  verbs: ["get", "watch", "list", "create", "update", "patch", "delete"]
# Required to create role and rolebinding to enable K8s discovery for TIBCO Streaming. 
- apiGroups: ["rbac.authorization.k8s.io"] 
  resources: ["clusterroles", "clusterrolebindings"]
  verbs: ["get", "list", "watch", "create", "update", "delete"]
# Allow to deploy configmaps for storing resources and configuration for TIBCO Streaming
- apiGroups: [""]
  resources: ["configmaps"]
  verbs: ["get", "watch", "list", "create", "update", "delete"]
# Allow to deploy stateful set of pods for TIBCO Streaming
- apiGroups: ["apps"]
  resources: ["statefulsets"]
  verbs: ["get", "watch", "list", "create", "update", "delete"]
- apiGroups: [""]
  resources: ["secrets"]
  verbs: ["create", "delete"]
!


# Role binding , can also do using k8s yaml
kubectl create clusterrolebinding $SERVICE_ACCOUNT:$SERVICE_ACCOUNT_ROLE --clusterrole $SERVICE_ACCOUNT_ROLE --serviceaccount default:$SERVICE_ACCOUNT

# Retrieve Kubernetes token
SECRET_NAME=$(kubectl get serviceaccount $SERVICE_ACCOUNT -n $NAMESPACE -o jsonpath='{.secrets[0].name}')
K8S_TOKEN=$(kubectl get secret $SECRET_NAME  -n $NAMESPACE -o jsonpath='{.data.token}' | base64 --decode)

echo Copy this token back into the Accelerator configuration setup
echo $K8S_TOKEN

