/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

/* Execute a GET against local router as proxy*/
var doGet = function(uri, resolve, reject) {
    $.ajax({
        type: 'GET',
        url: uri,
        dataType: 'json',
        cache: false,
        success: function(response, textStatus, xhr) {
            resolve(response);
        },
        error: function(xhr, textStatus, errorThrown) {
            reject(xhr);
        }
    });			
};


/* Application services */
app.service('ApplicationServices', function ($log, $rootScope) {
	return {
        // Retrieve environment from server
        getEnvironment: function() {
            var promise = new Promise(function(resolve, reject){
                var uri = BASE_URI + '/service/context/environment'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject(xhr);
                    }
                });        
            });

            return promise;
        },
        
        getApplicationConfiguration: function() {
            var promise = new Promise(function(resolve, reject){
                var uri = BASE_URI + '/service/context/configuration'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject(xhr);
                    }
                });
            });

            return promise;
        },

        saveApplicationConfiguration: function(configuration) {
            var promise = new Promise(function(resolve, reject){
                var uri = BASE_URI + '/service/context/configuration'; 
                var jsonData = JSON.stringify(configuration, null, 2);

                $.ajax({
                    type: 'POST',
                    url: uri,
                    dataType: 'json',
                    contentType: 'application/json',
                    data: jsonData,
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve();
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject();
                    }
                });        
            });

            return promise;
        },

        getRoutes: function() {
            var promise = new Promise(function(resolve, reject){
                // BASE_URI not required as this is loaded directly not via service call
                var uri = "assets/routes.json";

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject(xhr);
                    }
                });			
            });
            
            return promise;
        },

	};
});

/* Provisioning services */
app.service('ProvisioningServices', function($log, $rootScope){
    return {
        downloadSpotfireDXP: async function(oauth) {
            var url = BASE_URI + '/service/spotfire/analysis/' + main.region;

            // Hacky method to download a file from a link with a token
            function downloadWithJwtViaFormPost(url, id, token) {
                var jwtInput = $('<input type="hidden" name="jwtToken">').val(token);
                var idInput = $('<input type="hidden" name="id">').val(id);
                $('<form method="get" target="_blank"></form>')
                            .attr("action", url)
                            .append(jwtInput)
                            .append(idInput)
                            .appendTo('body')
                            .submit()
                            .remove();
            }
    
            // Call the download
            downloadWithJwtViaFormPost(url, 'my-id', oauth);    
        },

        loadScript: async function(scriptName) {
            var uri = BASE_URI + '/service/provisioning/script/' + scriptName;

            var promise = new Promise(function(resolve, reject){
                $.ajax({
                    type: 'GET',
                    url: uri,
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject(xhr);
                    }
                });						
            });

            return promise;    
        },

        provisionComponents: function(payload, onMessageCallback) {
            var promise = new Promise(function(resolve, reject){
                var uri = '/socket/provisioning/execute'
                
                var webSocketUri = "wss://" + location.host + location.pathname.replace('/app/iot/index.html', '') + uri;
                var socket = new WebSocket(webSocketUri);

                socket.addEventListener('open', function (event) {
                    socket.send(JSON.stringify(payload));
                });

                socket.addEventListener('message', function(event) {
                    onMessageCallback(event.data);
                });

                socket.addEventListener('close', function(event) {
                    resolve();
                });
            });
            
            return promise;
        },

    }
});

/* Simulation services */
app.service('SimulationServices', function($log, $rootScope){
    return {
        getTestCases: async function() {
            var uri = BASE_URI + '/service/simulation/testcases';

            var promise = new Promise(function(resolve, reject){
                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        var testcases = [];
                        for(var idx = 0; idx < response.testcases.length; idx++) {
                            var thisTestcase = response.testcases[idx].testcase;
                            if(thisTestcase != undefined) {
                                testcases.push(thisTestcase);
                            }
                        }
                        resolve(testcases);
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject(xhr);
                    }
                });					    
                });
            
            return promise;
        },

        sendCommand: async function(action, message) {
            var uri = BASE_URI + '/service/simulation/command/' + action;
    
            // Compose the payload and the send object
            var request = message;
            var jsonData = JSON.stringify(request, null, 2);

            var promise = new Promise(function(resolve, reject){
                $.ajax({
                    type: 'POST',
                    url: uri,
                    dataType: 'json',
                    contentType: 'application/json',
                    data: jsonData,
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve();
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        console.error(xhr);
                        reject();
                    }
                });			
    
            });
            
            return promise;    
        },

        subscribeStatus: function(payload, onMessageCallback) {
            var uri = BASE_URI + '/service/simulation/status'                

            var promise = new Promise(function(resolve, reject){
                var source = new EventSource(uri);
                source.addEventListener('message', function(message) {
                    onMessageCallback(JSON.parse(message.data));
                });                
            });
            
            return promise;
        }
    }
});

/* TIBCO Cloud services */
app.service('TibcoCloudServices', function ($log, $rootScope) {
	return {        
        whoami: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                // Default region to us (this is the only API call that does this)
                if(region == null) region = 'us';

                // Call proxy for the specified region
                var uri = BASE_URI + '/cic/account/' + region + '/tsc-ws/v2/whoami'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        if(oauth != null) {
                            xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                        }
                    },
                    success: function(response, textStatus, xhr) {
                        var result = {
                            authenticated: true,
                            identification: response
                        }
                        resolve(result);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        if(xhr.status == 401) {
                            var result = {
                                authenticated: false,
                                identification: null,
                                message: "Unauthorized"
                            }
                            resolve(result);
                        }
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        },

        loginCredentials: async function(region, username, password, tenantId, clientId) {
            var promise = new Promise(function(resolve, reject){
                //var uri = BASE_URI + '/cic/account/idm/v3/login-oauth'; 
                var uri = BASE_URI + '/service/account/' + region + '/login';
                var formData = {
                    Email: username,
                    Password: password,
                    TenantId: tenantId,
                    ClientID: clientId
                };

                $.ajax({
                    type: 'POST',
                    url: uri,
                    cache: false,
                    contentType: 'application/x-www-form-urlencoded',
                    dataType: 'json',
                    data: formData,
                    success: function(response, textStatus, xhr) {
                        var result = {
                            authenticated: true,
                            identification: response
                        }
                        resolve(result);
                    },
                    error: function(xhr, textStatus, errorThrown) {	
                        var result = {
                            authenticated: false,
                            identification: null,
                            message: xhr
                        }
                        reject(result);
                    }
                });
            });
            
            return promise;
        },

        logout: async function() {
            var promise = new Promise(function(resolve, reject){
                var uri = BASE_URI + '/service/account/logout'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    cache: false,
                    success: function(response, textStatus, xhr) {
                        resolve();
                    },
                    error: function(xhr, textStatus, errorThrown) {	
                        console.error(xhr);
                        reject();
                    }
                });    
            });
                
            return promise;
        },

        getDomain: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";

                var uri = BASE_URI + '/cic/account/' + region + '/tsc-ws/v1/tsc-domain'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        if(oauth != null) {
                            xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                        }
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	
                        console.error(xhr);
                        reject(xhr);
                    }
                });        
            });

            return promise;
        },

        getAccounts: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";

                var uri = BASE_URI + '/cic/account/' + region + '/tsc-ws/v1/my-accounts/info'; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        if(oauth != null) {
                            xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                        }
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	
                        console.error(xhr);
                        reject(xhr);
                    }
                });        
            });

            return promise;
        },

        getTcmChannels: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";
                if(oauth == null) throw "OAuth is required";

                var uri = BASE_URI + '/service/cic/oauth/datastreams/channels/' + region; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        },

        getTcmRoles: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";
                if(oauth == null) throw "OAuth is required";

                var uri = BASE_URI + '/service/cic/oauth/messaging/roles/' + region; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        },

        getTcmRoleConfiguration: async function(region, oauth, role) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";
                if(oauth == null) throw "OAuth is required";
                if(role == null) throw "Role is required";

                var uri = BASE_URI + '/service/cic/oauth/messaging/roleconfig/' + region + '/' + role; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        },

        getEventsSandboxes: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";
                if(oauth == null) throw "OAuth is required";

                var uri = BASE_URI + '/service/cic/oauth/events/sandboxes/' + region; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        },

        getDataStreams: async function(region, oauth) {
            var promise = new Promise(function(resolve, reject){
                if(region == null) throw "Region is required";
                if(oauth == null) throw "OAuth is required";

                var uri = BASE_URI + '/service/cic/oauth/datastreams/streams/' + region; 

                $.ajax({
                    type: 'GET',
                    url: uri,
                    dataType: 'json',
                    cache: false,
                    beforeSend: function(xhr) {
                        xhr.setRequestHeader('Authorization', 'Bearer ' + oauth);
                    },
                    success: function(response, textStatus, xhr) {
                        resolve(response);
                    },
                    error: function(xhr, textStatus, errorThrown) {	                        
                        console.error(xhr);
                        reject(xhr);
                    }
                });
        
            });
            return promise;
        }
 	};
});

/* Object services */
app.service('ObjectHelper', function ($log, $rootScope) {
	return {
        cloneObject: function(obj) {
            var clone = $.extend(true, {}, obj);
            delete clone._key;
            delete clone._source;
            return clone;
        },
	};
});